# -*- coding: utf-8 -*-
"""
@author: jbechtloff
Vorgabe: tsA, tsE, Timedelta

Zusammenführen von  sim_sig_design_03.py &
                    sim_mdb_struc_03.py
Generieren + Realtime                    
"""

from pymongo import MongoClient
import numpy as np
import pandas as pd
from PT1 import PT1
import time

import os
from apscheduler.schedulers.blocking import BlockingScheduler


class clProzess:
    def __init__(self,  T, T1, Kdrift, Tdrift, Kw, col):
        self.pt1a = PT1(T,T1)
        self.pt1b = PT1(T,T1)
        self.T = T
        self.Kdrift = Kdrift
        self.Tdrift = Tdrift
        self.Kw = Kw
        self.col = col
        self.cnt = 0

    def simProz(self):
        k = self.cnt
        u_1 = np.sin(4*k/100*np.pi)
        if u_1 <0: u_1=0
        if u_1 > 0.5: u_1 = 0.5
        u_1 = u_1*2.0

        # Amplituden-Drift-Simu: 1.0 < u2 < 1.1 (MUL)
        a_Drift = 1.0 + self.Kdrift * (np.sin(4*k*self.T/self.Tdrift*np.pi)+1.0)/2.0
        errFl = 0
        if a_Drift > 1.0+19*self.Kdrift/20: errFl = 1.0
        u_2 = self.pt1b.PT1rek(self.pt1a.PT1rek(u_1*a_Drift))  # PT2-Verschleiß
        w = np.random.randn()*self.Kw

        self.cnt +=1
        return(u_1, u_2, a_Drift, w, errFl)

    def write_mdb(self, ts):
        [u_1, u_2, a_Drift, wi, err_Flg] = self.simProz()

        data = {
            'ts': ts,
            'cnt': self.cnt,
            'u': u_2 + wi,
            'err': err_Flg
        }
        result = self.col.insert_one(data)
        print("%s" % (self.cnt))

    def write_mdb_now(self):
        self.write_mdb(pd.Timestamp.now())


if __name__ == "__main__":
    client = MongoClient('localhost', 27017)
    db = client['IoT']
    col = db["m"]
    col.drop()  # Alte Collection LÖSCHEN !!!

    Prozess = clProzess(T=1,T1=4,
                    Kdrift=0.1, 
                    Tdrift=2000, 
                    Kw=0.01,
                    col = col)

    
    
    Tscal = {"s":1,"m":60,"h":60*60, "d":60*60*24}
    dT = 1
    T_unit = "s"

    sched = BlockingScheduler({'apscheduler.timezone': 'UTC'})
    sched.add_job(Prozess.write_mdb_now,'interval',seconds=Tscal[T_unit]*dT,id='runJob1') 
    
    # tsA = pd.Timestamp(2021,11, 20,  0,0,0)
    # tsE = pd.Timestamp(2021,11, 21,  10,0,0)

    tsE = pd.Timestamp.now()  # now
    tsA = tsE - pd.Timedelta(1,"h")
    # tsDeltaAE = tsE - tsA

    
    ts = tsA
    while ts <= tsE: 
        Prozess.write_mdb(ts)
        ts = ts + pd.Timedelta(dT,T_unit)


    print('Press Ctrl+{0} to exit'.format('Break' if os.name == 'nt' else 'C'))

    try:
        sched.start()
    except (KeyboardInterrupt, SystemExit):
        pass                

